"""
WSGI config for motty project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/howto/deployment/wsgi/
"""

import os
import sys

from django.core.wsgi import get_wsgi_application

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(BASE_DIR)
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "motty.settings")

application = get_wsgi_application()



import os
import tornado.httpserver
import tornado.ioloop
import tornado.wsgi
import sys
from django.core.wsgi import get_wsgi_application

libpath = [path for path in sys.path if 'site-packages' in path][0]
sys.path.append(libpath + "/motty")

def run_motty():
    STATIC_ROOT = libpath + '/motty/app/static'

    os.environ['DJANGO_SETTINGS_MODULE'] = 'motty.settings' # path to your settings module
    application = get_wsgi_application()
    container = tornado.wsgi.WSGIContainer(application)

    tornado_app = tornado.web.Application([
        (r'/static/(.*)', tornado.web.StaticFileHandler, { 'path':STATIC_ROOT }),
        (r'.*', tornado.web.FallbackHandler, dict(fallback=container))
    ])

    http_server = tornado.httpserver.HTTPServer(tornado_app)
    http_server.listen(8888)
    tornado.ioloop.IOLoop.instance().start()

run_motty();


# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-12 04:12
from __future__ import unicode_literals

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Action',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('url', models.CharField(max_length=50)),
                ('method', models.CharField(max_length=50)),
                ('contentType', models.CharField(max_length=100)),
                ('body', models.TextField()),
                ('created_at', models.DateTimeField(default=datetime.datetime(2017, 11, 12, 4, 12, 48, 881343))),
            ],
        ),
    ]



from rest_framework.decorators import api_view

from django.shortcuts import render, redirect
from django.http import HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt

from django.forms.models import model_to_dict
from django.db import connection
from django.core import serializers
from django.core.exceptions import ObjectDoesNotExist

from .utils import remove_last_slash
from .models import Action
from .serializers import ActionSerializer

import json

# main view
def main(request):
    return redirect(index)

def index(request):
    return render(request, 'app/index.html')

# api requests.
@api_view(['GET'])
def actions(request):
    actions = Action.objects.values()
    serializer = ActionSerializer(actions, many=True)
    return JsonResponse(serializer.data, safe=False);

@csrf_exempt
@api_view(['POST'])
def create_new_action(request):
    data = json.loads(request.body.decode('utf-8'))
    serializer = ActionSerializer(data=data)
    if serializer.is_valid() :
        action = serializer.save()
        return JsonResponse(serializer.data, safe=False)
    else:
        return JsonResponse(serializer.errors, status=400)

@csrf_exempt
@api_view(['POST'])
def edit_action(request, id):
    data = json.loads(request.body.decode('utf-8'))
    serializer = ActionSerializer(data=data)
    if serializer.is_valid() :
        _action = serializer.data
        try:
            action = Action.objects.get(pk=id)
            action.name = _action['name']
            action.url = _action['url']
            action.method = _action['method']
            action.contentType = _action['contentType']
            action.body = _action['body']
            action.save()
            return HttpResponse('data is successfully saved.')
        except ObjectDoesNotExist:
            return HttpResponse('There is no data to udpate.', status=404)
    else:
        return JsonResponse(serializer.errors, status=400)

@api_view(['GET'])
def delete_action(request, id):
    try:
        action = Action.objects.get(pk=id)
        action.delete()
        return HttpResponse('deleted.')
    except ObjectDoesNotExist:
        return HttpResponse('There is no data to delete.')

@csrf_exempt
@api_view(['POST'])
def delete_all(request):
    res = json.loads(request.body.decode('utf-8'))
    ids = res['ids']
    with connection.cursor() as cursor:
        cursor.execute('DELETE FROM app_action WHERE id in ' + "(" + ",".join(map(str, ids)) + ")")

    return HttpResponse("your requests are successfully conducted.")


@api_view(['GET'])
def get_action(request, id):
    action = Action.objects.get(pk=id)
    return JsonResponse(model_to_dict(action))

# fake responses.
@csrf_exempt
def return_fake_request(request, endpoint):
    endpoint = "/" + endpoint
    actions = Action.objects.filter(url=remove_last_slash(endpoint), method=request.method)
    if len(actions) > 0:
        action = actions[0]
        return HttpResponse(action.body, content_type=action.contentType)
    else:
        return HttpResponse('No such response exists.', status=404)


from django.conf.urls import url
from . import views

urlpatterns = [
    url(r'^app/$', views.index, name = "index"),
    url(r'^app/api/actions', views.actions, name = "actions"),
    url(r'^app/api/action$', views.create_new_action, name = "create_new_action"),
    url(r'^app/api/action/([0-9]{1,})$', views.get_action, name = "get_action"),
    url(r'^app/api/action/([0-9]{1,})/edit$', views.edit_action, name = "edit_action"),
    url(r'^app/api/action/([0-9]{1,})/delete$', views.delete_action, name = "delete_action"),
    url(r'^app/api/action/delete/all$', views.delete_all, name = "delete_all"),
    url(r'^motty/base/(.+)', views.return_fake_request, name = "return_fake_request"),
    url(r'', views.main, name = "main"),
]


from .models import Action
from .utils import remove_last_slash
from rest_framework import serializers

import datetime

class ActionSerializer(serializers.ModelSerializer):
    class Meta:
        model = Action
        fields = ('id', 'name', 'url', 'method', 'contentType', 'body', 'created_at')

    def create(self, validated_data):
        validated_data['url'] = remove_last_slash(validated_data['url'])
        return Action.objects.create(**validated_data)

def json_serializer(obj):
    """Default JSON serializer."""

    if isinstance(obj, datetime.datetime):
        return obj.strftime('%Y-%m-%d %H:%M:%S.%f')
    return str(obj)


from datetime import datetime
from django.db import models

class Action(models.Model):
    '''A Response, I\'ll group actions to the resource at near future. '''
    name = models.CharField(max_length=30)
    url = models.CharField(max_length=50)
    method = models.CharField(max_length=50)
    contentType = models.CharField(max_length=100)
    body = models.TextField()
    created_at = models.DateTimeField(default = datetime.now())

    def __str__(self):
        return "name: {0}, url: {1}, method: {2}, contentType: {3}, body: {4}, created_at: {5}"\
            .format(self.name, self.url, self.method, self.contentType, self.body, self.created_at)



from setuptools import find_packages, setup

setup(
  name = 'motty',
  packages = ['motty'], # this must be the same as the name above
  version = '0.1',
  description = 'utils for http mocking',
  author = 'David Lee',
  author_email = 'scalalang2@gmail.com',
  include_package_data = True,
  scripts=['motty/runmotty.py'],
  entry_points={'console_scripts': [
    'run-motty = runmotty:run_motty',
  ]},
  install_requires=['django', 'djangorestframework', 'django-libsass', 'libsass', 'django-compressor',
    'django-sass-processor', 'tornado'],
  url = 'https://github.com/scalalang2/motty', # use the URL to the github repo
  download_url = 'https://github.com/scalalang2/motty/archive/0.1.tar.gz', # I'll explain this in a second
  keywords = ['mocking', 'motty', 'http'], # arbitrary keywords
  classifiers=[ 
  ],
)
